"""
android-sdk-updater

:copyright: (c) 2016 Tad Fisher
:license: Apache 2.0, see LICENSE for more details.

"""

__version__ = '1.0.1'
__author__ = 'Tad Fisher'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2016 Tad Fisher'
